const _ = require("lodash");

const {Step} = require("cattura-ui-library/src/lib/ui/wizard");
const ko = require("knockout");

function makeStep (options)
{
    const templates = ko.computed(() => {
         const recorder = ko.unwrap(options.recorder) || {};
         return ko.unwrap(recorder.templates) || [];
    });
    
    let title = ko.observable();
    let template = ko.observable();
    let appendDateToTitle = ko.observable();
    let tags = ko.observableArray();

    function reset ()
    {
        title("");
        template(null);
        appendDateToTitle(false);
        tags([]);
    }

    function populate (event)
    {
        title(event.title);
        template(_(templates()).find(template => template.id === event.template));
        tags(event.tags);
    }

    function complete ()
    {
        return title() !== "" && template();
    }

    return Step({
        step: 2,
        title: title,
        recorder: options.recorder,
        recorders: options.recorders,
        template: template,
        templates: templates,
        appendDateToTitle: appendDateToTitle,
        tags: tags,
        populate: populate,
        reset: reset,
        complete: complete
    });    
}

module.exports = makeStep;