const _ = require("lodash");
const {Step} = require("cattura-ui-library/src/lib/ui/wizard");
const ko = require("knockout");
const moment = require("moment");

const DATE_FORMAT = "YYYY-MM-DD";
const TIME_FORMAT = "hh:mm A";

function makeStep (options)
{
    const momentGenerator = options.momentGenerator || moment;
    
    function dateTimeObservable (date, time)
    {
        return ko.computed({
            read ()
            {
                const dateMoment = momentGenerator(date(), DATE_FORMAT);
                const timeMoment = momentGenerator(time(), TIME_FORMAT);

                return dateMoment.set({
                    hour: timeMoment.hour(),
                    minute: timeMoment.minute(),
                    second: timeMoment.second()
                });
            },
            write (value)
            {
                value = momentGenerator(value);
                date(value.format(DATE_FORMAT));
                time(value.format(TIME_FORMAT));
            }
        });
    }    
    
    let date = ko.observable();
    let time = ko.observable();
    const dateTime = dateTimeObservable(date, time);

    let duration = ko.observable(moment.duration(0, "hours"));
    let textDuration = ko.computed({
        read ()
        {
            return duration().format("hh:mm", { trim: false });
        },
        write (value)
        {
            duration(moment.duration(value));
        }
    }).extend({notify: "always"});

    let recur = ko.observable(false);
    let recurrenceStartDate = ko.observable(); // Only required for existing recurrences. New recurrences start at the recording's date
    let recurrenceEndDate = ko.observable();

    const recurrenceStartDateTime = dateTimeObservable(recurrenceStartDate, time);

    let days = {
        monday:    ko.observable(false),
        tuesday:   ko.observable(false),
        wednesday: ko.observable(false),
        thursday:  ko.observable(false),
        friday:    ko.observable(false),
        saturday:  ko.observable(false),
        sunday:    ko.observable(false)
    };

    function reset ()
    {
        const now = momentGenerator().add(1, "hours").minutes(0);
        dateTime(now);
        duration(moment.duration(1, "hours"));

        recur(false);
        recurrenceStartDate(null);
        recurrenceEndDate(now.clone().add(1, "days"));
        _(days).each(flag => flag(false));
    }

    function populate (event)
    {
        dateTime(momentGenerator(event.startTime));
        duration(moment.duration(event.stopTime - event.startTime));

        if (event.group)
        {
            recur(true);
            recurrenceStartDate(momentGenerator(event.group.startTime).format(DATE_FORMAT));
            recurrenceEndDate(momentGenerator(event.group.stopTime));
            _(days).values()
                   .filter((flag, index) => _(event.group.days).includes(index))
                   .each((flag, index) => flag(true));
        }
    }

    function complete ()
    {
        if (recur() && (!recurrenceEndDate() || !_(days).find(flag => flag())))
        {
            return false;
        }

        return date() && time() && duration().asMilliseconds();
    }

    return Step({
        step: 3,
        dateTime: dateTime,
        date: date,
        time: time,
        duration: duration,
        textDuration: textDuration,

        recur: recur,
        recurrenceStartDate: recurrenceStartDate,
        recurrenceStartDateTime: recurrenceStartDateTime,
        recurrenceEndDate: recurrenceEndDate,
        days: days,

        populate: populate,
        reset: reset,
        complete: complete
    });    
}

module.exports = makeStep;