const _ = require("lodash");
const ko = require("knockout");
const uuidv4 = require("uuid/v4");

const FileUploader = require("cattura-ui-library/src/lib/ui/fileUploader");
const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");

const ASSET_CLASS = "com.cattura.share.processor.data_provider.elements.Asset";
const LINK_CLASS = "com.cattura.share.processor.data_provider.elements.Link";

function makeDialog (options)
{
    let assetId = ko.observable();

    let assetName = ko.observable().extend({
        required: true
    });

    let type = ko.observable("file");

    let url = ko.observable().extend({
        required: {
            onlyIf: () => type() === "link"
        }
    });
    let filePath = ko.observable().extend({
        required: {
            onlyIf: () => type() === "file"
        }
    });
    let fileName = ko.observable().extend({
        required: {
            onlyIf: () => type() === "file"
        }
    });

    const uploader = ko.computed(() => FileUploader({
        endpoint: (ko.unwrap(options.recorder) || {}).basePath + "api/1/upload/files/assets"
    }));

    return Dialog({
        id: assetId,

        fields: ko.validatedObservable({
            name: assetName,
            type: type,
            url: url,
            path: filePath,
            fileAlias: fileName
        }),

        open (asset)
        {
            this.reset();

            if (asset && "@class" in asset)
            {
                this.id(asset.id);
                this.fields().name(asset.name);
                this.fields().url(asset.url);
                this.fields().path(asset.path);
                this.fields().fileAlias(asset.fileAlias);

                if (asset["@class"] === ASSET_CLASS)
                {
                    this.fields().type("asset");
                }
                else
                {
                    this.fields().type("link");
                }
            }

            this.show();
        },

        reset ()
        {
            this.id("");
            this.fields().name("");
            this.fields().type("file");
            this.fields().url("");
            this.fields().path("");
            this.fields().fileAlias("");
        },

        uploadFile ()
        {
            uploader().fileChooser().then(result => {
                this.fields().fileAlias(result.file.name);
                this.fields().path(result.file.path);
            });
        },

        save ()
        {
            if (!this.fields.isValid())
            {
                this.dispatchEvent({type: "error", message: "Form is not valid."});
                return;
            }

            let asset = {
                id: this.id() || `speaker:${uuidv4()}`,
                name: this.fields().name()
            };

            if (this.fields().type() === "file")
            {
                _(asset).extend({
                    "@class": ASSET_CLASS,
                    mimeType: "application/x-octet-stream",
                    path: this.fields().path(),
                    fileAlias: this.fields().fileAlias()
                }).value();
            }
            else
            {
                _(asset).extend({
                    "@class": LINK_CLASS,
                    title: this.fields().name(),
                    url: this.fields().url()
                }).value();
            }

            this.dispatchEvent({type: "assetsaved", asset: asset});
            this.hide();
        }
    });    
}

module.exports = makeDialog;
