const ko = require("knockout");
const _ = require("lodash");
const toastr = require("toastr");

const {Step} = require("cattura-ui-library/src/lib/ui/wizard");
const ContextMenu = require("cattura-ui-library/src/lib/ui/contextMenu");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");

/* jshint -W079 */
const confirm = require("cattura-ui-library/src/modals/confirm");

function makeStep (options)
{
    const addEditAsset = require("./add-edit-asset")(options);
    addEditAsset.addEventListener("assetsaved", event => {
        let newAsset = selectable.makeSelectable(extendAsset(event.asset));
        let newAssets = _(assets()).map(asset => {
            if (newAsset && asset.id === newAsset.id)
            {
                asset = newAsset;
                newAsset = null;
            }

            return asset;
        }).value();

        if (newAsset)
        {
            newAssets.push(newAsset);
        }

        assets(newAssets);
        toastr.success(`Asset "${event.asset.name}" saved successfully.`);
    });

    let assets = selectable.selectableObservableArray();

    function extendAsset (asset)
    {
        asset.shortType = () => asset["@class"].substring(asset["@class"].lastIndexOf(".") + 1);
        asset.downloadLink = () => "api/1/upload/" + (asset.url || asset.path);
        return asset;
    }

    function reset ()
    {
        assets([]);
    }

    function populate (event)
    {
        assets(_(event.assets).chain().cloneDeep().map(selectable.makeSelectable).map(extendAsset).value());
    }

    function complete ()
    {
        return true;
    }

    const actions = ContextMenu({
        edit (asset)
        {
            addEditAsset.open(asset);
        },

        deleteAsset (asset)
        {
            confirm.open(`Delete asset ${asset.name}?`).then(() => {
                assets(_(assets()).without(asset).value());
                toastr.success(`Asset "${asset.name}" deleted successfully.`);
            });
        }
    });

    function deleteSelected ()
    {
        assets(_(assets()).reject(asset => asset.selected()).value());
    }

    return Step({
        step: 4,
        assets: assets,
        reset: reset,
        populate: populate,
        complete: complete,
        actions: actions,
        deleteSelected: deleteSelected,
        addEditAsset: addEditAsset
    });    
}

module.exports = makeStep;