const _ = require("lodash");
const ko = require("knockout");
const toastr = require("toastr");

const ContextMenu = require("cattura-ui-library/src/lib/ui/contextMenu");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");

const Exclusion = require("../tab/exclusion.js");
const {apiv1} = require("./api");
const {makeMoment} = require("./time");

/* jshint -W079 */
const confirm = require("cattura-ui-library/src/modals/confirm");

function extendExclusion (exclusion)
{
    exclusion.startDate = makeMoment(exclusion.startDate).utc();
    exclusion.endDate = makeMoment(exclusion.endDate).utc();
    return exclusion;
}

function createBlackoutSettings (plugin)
{
    const addEditExclusion = require("./add-edit-exclusion")(plugin);
    addEditExclusion.addEventListener("exclusionsaved", event => {
        toastr.success(`Exclusion "${event.exclusion.label}" saved successfully.`);
        plugin.tab.loadExclusions();
    });
    addEditExclusion.addEventListener("error", event => toastr.error(event.message));
    
    let exclusions = selectable.selectableObservableWrapper(() => {
        return _(plugin.tab.exclusions())
            .map(Exclusion)
            .map(extendExclusion)
            .map(selectable.makeSelectable)
            .value();
    });

    function deleteSelected ()
    {
        confirm.open("Delete all selected blackout dates?").then(() => {
            return Promise.all(_(exclusions.selected()).map(exclusion => {
                return plugin.services.removeExclusion({
                    id: exclusion.id
                });
            }).value());
        }).then(() => {
            toastr.success("Blackout dates deleted successfully.");
            plugin.tab.loadExclusions();
        }).catch(error => {
            toastr.error(error.message);
        });
    }

    const actions = ContextMenu({
        edit (exclusion)
        {
            addEditExclusion.open(exclusion);
        },

        deleteExclusion (exclusion)
        {
            confirm.open(`Delete blackout date ${exclusion.label}?`).then(() => {
                return plugin.services.removeExclusion({
                    id: exclusion.id
                });
            }).then(() => {
                toastr.success(`Blackout date "${exclusion.label}" deleted successfully.`);
                plugin.tab.loadExclusions();
            }).catch(error => {
                toastr.error(error.message);
            });
        }
    });

    function load ()
    {
        plugin.tab.loadExclusions();
    }

    return {
        addEditExclusion: addEditExclusion,
        exclusions: exclusions,
        actions: actions,
        deleteSelected: deleteSelected,
        load: load,
        render: require("./content.pug")
    };    
}

module.exports = createBlackoutSettings;