const $ = require("jquery");
const _ = require("lodash");
const ko = require("knockout");

const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const {Wizard, Step} = require("cattura-ui-library/src/lib/ui/wizard");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");
const FileUploader = require("cattura-ui-library/src/lib/ui/fileUploader");

const MediaPackage = require("cattura-ui-library/src/lib/mediaPackage");
const {apiv1} = require("./api");

const template = require("./content.pug");

let uri = ko.observable();
let error = ko.observable();

const uploader = FileUploader({
    endpoint: "/api/1/upload/files/calendar"
});

function clear ()
{
    uri("");
    error(null);
}

let resolver = null;

function makeDialog (plugin)
{
    const submitEvents = require("./submit-events")(plugin);
    
    function importCalendar (calendarUri)
    {
        return plugin.services.parseCalendarFile({
            calendarFile: calendarUri
        }).then(result => {
            dialog.hide();
            return submitEvents.open(result).then(events => {
                dialog.dispatchEvent({type: "calendarimported", events: events});
                return events;
            });
        }).then(events => {
            if (resolver)
            {
                return resolver(events);
            }
            
            return events;
        });
    }

    function importCalendarFromURI ()
    {
        const calendarUri = uri();
        if (!calendarUri)
        {
            return;
        }

        return importCalendar(calendarUri);
    }

    function uploadFileAndImport ()
    {
        return uploader.fileChooser().then(result => {
            return importCalendar(result.file.path);
        });
    }

    const dialog = Dialog({
        open: function open ()
        {
            clear();
            this.show();
            $('.import-recs-url-input').focus();
            return new Promise((resolve, reject) => resolver = resolve);
        },

        uri: uri,
        importCalendarFromURI: importCalendarFromURI,
        uploadFileAndImport: uploadFileAndImport,
        error: error,

        submitEvents: submitEvents
    }, {
        template: template
    });
    
    return dialog;
}

module.exports = makeDialog;
