const _ = require("lodash");
const ko = require("knockout");
const Dialog = require("cattura-ui-library/src/lib/ui/dialog");
const selectable = require("cattura-ui-library/src/lib/ui/selectable");
const moment = require("moment");

const templates = require("./templates").templates;

let importedEvents = selectable.selectableObservableArray();
let batches = ko.observableArray();
let template = ko.observable();
let error = ko.observable();

const dialogTemplate = require("./content.pug");

let resolver = null;

const allImportableEvents = ko.computed(() => {
    return _(batches()).map(batch => {
        return _(batch.events).map(event => {
            return {
                event: _({}).extend(event.event).extend({
                    title: event.event.title()
                }).value(),
                task: {
                    templateID: batch.template.id
                }
            };
        }).value();
    }).flatten().value();
});

function extendScheduledEvent (event)
{
    event.isEditing = ko.observable(false);
    event.event.title = ko.observable(event.event.title);
    event.event.duration = moment.duration(event.event.stopTime - event.event.startTime);
    
    const toggle = event.toggleSelected;
    event.toggleSelected = function toggleSelected ()
    {
        if (!this.canBeSubmitted())
        {
            return;
        }
        
        toggle.call(this);
    };
    
    event.batched = ko.observable(false);
    event.canBeSubmitted = function canBeSubmitted ()
    {
        return event.event.startTime > new Date().getTime();
    };
    
    return event;
}

function clear ()
{
    importedEvents([]);
    batches([]);
    error(null);
}

function submitBatch ()
{
    batches(_(batches()).concat({
        template: template(),
        events: _(importedEvents.selected()).map(event => {
            event.selected(false);
            event.batched(true);
            return event;
        }).value()
    }).value());
}

function removeBatch (batch)
{
    _(batch.events).forEach(event => event.batched(false));
    batches(_(batches()).without(batch).value());
}

function makeDialog (plugin)
{
    function importAllBatches ()
    {
        const events = allImportableEvents();
        plugin.services.scheduleEvents({events: events}).then(retval => {
            resolver(events);
            dialog.hide();
        });
    }

    const dialog = Dialog({    
        open (events)
        {
            clear();
            importedEvents(_(events).map(selectable.makeSelectable).map(extendScheduledEvent).value());
            this.show();
            return new Promise((resolve, reject) => resolver = resolve);
        },

        error: error,
        importedEvents: importedEvents,
        batches: batches,
        template: template,
        templates: templates,
        submitBatch: submitBatch,
        removeBatch: removeBatch,
        importAllBatches: importAllBatches,
        allImportableEvents: allImportableEvents
    }, {
        template: dialogTemplate
    });
    
    return dialog;
}

module.exports = makeDialog;
